@@ -37,6 +37,7 @@ from ast import (
     Module,
     Mult,
     Name,
+    NamedExpr,
     NodeTransformer,
     NodeVisitor,
     Pass,
@@ -45,7 +46,6 @@ from ast import (
     RShift,
     Starred,
     Store,
-    Str,
     Sub,
     Subscript,
     Tuple,
@@ -65,9 +65,6 @@ from copy import deepcopy
 from dataclasses import dataclass, field
 from typing import Any, ClassVar, cast, overload
 
-if sys.version_info >= (3, 8):
-    from ast import NamedExpr
-
 generator_names = (
     "typing.Generator",
     "collections.abc.Generator",
@@ -159,13 +156,12 @@ class TransformMemo:
                 if isinstance(child, ImportFrom) and child.module == "__future__":
                     # (module only) __future__ imports must come first
                     continue
-                elif isinstance(child, Expr):
-                    if isinstance(child.value, Constant) and isinstance(
-                        child.value.value, str
-                    ):
-                        continue  # docstring
-                    elif sys.version_info < (3, 8) and isinstance(child.value, Str):
-                        continue  # docstring
+                elif (
+                    isinstance(child, Expr)
+                    and isinstance(child.value, Constant)
+                    and isinstance(child.value.value, str)
+                ):
+                    continue  # docstring
 
                 self.code_inject_index = index
                 break
@@ -379,6 +375,12 @@ class AnnotationTransformer(NodeTransformer):
         self.generic_visit(node)
 
         if isinstance(node.op, BitOr):
+            # If either branch of the BinOp has been transformed to `None`, it means
+            # that a type in the union was ignored, so the entire annotation should e
+            # ignored
+            if not hasattr(node, "left") or not hasattr(node, "right"):
+                return None
+
             # Return Any if either side is Any
             if self._memo.name_matches(node.left, *anytype_names):
                 return node.left
@@ -411,7 +413,7 @@ class AnnotationTransformer(NodeTransformer):
         # don't try to evaluate it as code
         if node.slice:
             if isinstance(node.slice, Index):
-                # Python 3.7 and 3.8
+                # Python 3.8
                 slice_value = node.slice.value  # type: ignore[attr-defined]
             else:
                 slice_value = node.slice
@@ -422,20 +424,22 @@ class AnnotationTransformer(NodeTransformer):
                     # forward reference
                     items = cast(
                         typing.List[expr],
-                        [self.generic_visit(slice_value.elts[0])]
-                        + slice_value.elts[1:],
+                        [self.visit(slice_value.elts[0])] + slice_value.elts[1:],
                     )
                 else:
                     items = cast(
                         typing.List[expr],
-                        [self.generic_visit(item) for item in slice_value.elts],
+                        [self.visit(item) for item in slice_value.elts],
                     )
 
                 # If this is a Union and any of the items is Any, erase the entire
                 # annotation
                 if self._memo.name_matches(node.value, "typing.Union") and any(
-                    isinstance(item, expr)
-                    and self._memo.name_matches(item, *anytype_names)
+                    item is None
+                    or (
+                        isinstance(item, expr)
+                        and self._memo.name_matches(item, *anytype_names)
+                    )
                     for item in items
                 ):
                     return None
@@ -491,15 +495,6 @@ class AnnotationTransformer(NodeTransformer):
 
         return node
 
-    def visit_Str(self, node: Str) -> Any:
-        # Only used on Python 3.7
-        expression = ast.parse(node.s, mode="eval")
-        new_node = self.visit(expression)
-        if new_node:
-            return copy_location(new_node.body, node)
-        else:
-            return None
-
 
 class TypeguardTransformer(NodeTransformer):
     def __init__(
@@ -511,11 +506,32 @@ class TypeguardTransformer(NodeTransformer):
         self.target_node: FunctionDef | AsyncFunctionDef | None = None
         self.target_lineno = target_lineno
 
+    def generic_visit(self, node: AST) -> AST:
+        has_non_empty_body_initially = bool(getattr(node, "body", None))
+        initial_type = type(node)
+
+        node = super().generic_visit(node)
+
+        if (
+            type(node) is initial_type
+            and has_non_empty_body_initially
+            and hasattr(node, "body")
+            and not node.body
+        ):
+            # If we have still the same node type after transformation
+            # but we've optimised it's body away, we add a `pass` statement.
+            node.body = [Pass()]
+
+        return node
+
     @contextmanager
     def _use_memo(
         self, node: ClassDef | FunctionDef | AsyncFunctionDef
     ) -> Generator[None, Any, None]:
         new_memo = TransformMemo(node, self._memo, self._memo.path + (node.name,))
+        old_memo = self._memo
+        self._memo = new_memo
+
         if isinstance(node, (FunctionDef, AsyncFunctionDef)):
             new_memo.should_instrument = (
                 self._target_path is None or new_memo.path == self._target_path
@@ -567,8 +583,6 @@ class TypeguardTransformer(NodeTransformer):
         if isinstance(node, AsyncFunctionDef):
             new_memo.is_async = True
 
-        old_memo = self._memo
-        self._memo = new_memo
         yield
         self._memo = old_memo
 
@@ -605,8 +619,9 @@ class TypeguardTransformer(NodeTransformer):
         return node
 
     def visit_Module(self, node: Module) -> Module:
+        self._module_memo = self._memo = TransformMemo(node, None, ())
         self.generic_visit(node)
-        self._memo.insert_imports(node)
+        self._module_memo.insert_imports(node)
 
         fix_missing_locations(node)
         return node
@@ -705,14 +720,12 @@ class TypeguardTransformer(NodeTransformer):
                 if self.target_lineno == first_lineno:
                     assert self.target_node is None
                     self.target_node = node
-                    if node.decorator_list and sys.version_info >= (3, 8):
+                    if node.decorator_list:
                         self.target_lineno = node.decorator_list[0].lineno
                     else:
                         self.target_lineno = node.lineno
 
-                all_args = node.args.args + node.args.kwonlyargs
-                if sys.version_info >= (3, 8):
-                    all_args.extend(node.args.posonlyargs)
+                all_args = node.args.args + node.args.kwonlyargs + node.args.posonlyargs
 
                 # Ensure that any type shadowed by the positional or keyword-only
                 # argument names are ignored in this function
@@ -909,6 +922,22 @@ class TypeguardTransformer(NodeTransformer):
 
                 self._memo.insert_imports(node)
 
+                # Special case the __new__() method to create a local alias from the
+                # class name to the first argument (usually "cls")
+                if (
+                    isinstance(node, FunctionDef)
+                    and node.args
+                    and self._memo.parent is not None
+                    and isinstance(self._memo.parent.node, ClassDef)
+                    and node.name == "__new__"
+                ):
+                    first_args_expr = Name(node.args.args[0].arg, ctx=Load())
+                    cls_name = Name(self._memo.parent.node.name, ctx=Store())
+                    node.body.insert(
+                        self._memo.code_inject_index,
+                        Assign([cls_name], first_args_expr),
+                    )
+
                 # Rmove any placeholder "pass" at the end
                 if isinstance(node.body[-1], Pass):
                     del node.body[-1]
@@ -1190,11 +1219,6 @@ class TypeguardTransformer(NodeTransformer):
         """
         self.generic_visit(node)
 
-        # Fix empty node body (caused by removal of classes/functions not on the target
-        # path)
-        if not node.body:
-            node.body.append(Pass())
-
         if (
             self._memo is self._module_memo
             and isinstance(node.test, Name)
